--[[ Pricing

     Check the pricing on particular items.

]]--

local default_items = {
  blue = "I03F32B00FB60EB5B,EF5F7E9F60BAC7B6,,,,,,",
  bluekey = "I5E109954596019F6,5932D4C037641DF0,,,,,,",
  key = "I174321237F7F33A4,3031DFD5B66AC285,,,,,,",
  orange = "I3C0FD5A697D50095,7B6543539C3CF2F0,,,,,,",
  orangekey = "I5E109956C75E8F55,EF5695F666490E6E,,,,,,",
  purple = "IFD9DDA7BA84BDDB4,3B1787C715044625,,,,,,",
  purplekey = "I5E10995561A6164C,4E50FB6B4789ADDF,,,,,,",
  rex = "I0000002A83B9AD92,4A38F083DE39D173,,,,,,",
  skelkey = "I174321237F7F33A4,3031DFD5B66AC285,,,,,,"
}

local default_channels = {
  '3', '4', '5', '6',
}

local default_message = "Recent AH averages for Keys/boxes/REX: Keys {bluekey}/{purplekey}/{orangekey}, boxes {blue}/{purple}/{orange}, REX {rex}."

local addoninfo, pricing = ...
local pgc = Library.LibPGC
local printf = Library.printf.printf
local sprintf = Library.printf.sprintf
local floor = math.floor
local ceil = math.ceil

pricing.buttons = {}

pricing.seen = {
}

pricing.last_computed = nil

function pricing.newbutton(idx, channel)
  local button = UI.CreateFrame('RiftButton', sprintf('list pricing on /%s', channel), pricing.buttoncontext)
  button:SetVisible(false)
  button:SetPoint("BOTTOMLEFT", pricing.updatebutton, "BOTTOMLEFT", (1 - (idx % 2)) * 50, ceil(idx / 2) * 30)
  button:SetSecureMode("restricted")
  button:SetWidth(50)
  local text = UI.CreateFrame('Text', 'button label',  button)
  text:SetPoint("CENTER", button, "CENTER")
  text:SetText(channel)
  pricing.buttons[channel] = button
  button:SetVisible(true)
  return button
end

function pricing.variables(event, addon)
  if addon == 'Pricing' then
    Pricing_sv = Pricing_sv or { }
    Pricing_sv.loc = Pricing_sv.loc or { x = 0, y = 500 }
    Pricing_sv.items = Pricing_sv.items or default_items
    Pricing_sv.message = Pricing_sv.message or default_message
    Pricing_sv.channels = Pricing_sv.channels or default_channels
  end
end

function pricing.pretty(n)
  return sprintf("%dp", n / 10000)
end

function pricing.prettyverbose(n, prefix)
  local components = {}
  local s = n % 100
  n = (n - s) / 100
  local g = n % 100
  local p = (n - g) / 100
  if n == 0 then
    return '<font color="#888888">No value.</font>'
  end
  if prefix then
    components[#components + 1] = '<font color="#888888">' .. prefix .. ':</font> '
  end
  if p > 0 then
    components[#components + 1] = sprintf('<font color="#ddddff">%dp</font>', p)
  end
  if g > 0 or p > 0 then
    if g > 0 then
      components[#components + 1] = sprintf('<font color="#bbaa22">%dg</font>', g)
    else
      components[#components + 1] = sprintf('<font color="#998811">0g</font>', g)
    end
  end
  if s > 0 or g > 0 or p > 0 then
    if s > 0 then
      components[#components + 1] = sprintf('<font color="#dddddd">%ds</font>', s)
    else
      components[#components + 1] = sprintf('<font color="#999999">0s</font>')
    end
  end
  return table.concat(components, "")
end

function pricing.display_update(key, data)
  if data and data.BMarket and data.BMarket.buy then
    local details = pricing.pretty(data.BMarket.buy)
    printf("Suggested price for %s: %s", key, details)
  else
    printf("No known price for %s.", key)
  end
end

function pricing.get_update(key, data)
  if data and data.BMarket and data.BMarket.buy then
    pricing.seen[key] = data.BMarket.buy
  end
  -- printf("Update for %s", key)
  local newmessage = Pricing_sv.message
  local problems = {}
  while newmessage:find("{[^}]*}") do
    local first, last = newmessage:find("{[^}]*}")
    local before = newmessage:sub(1, first - 1)
    local during = newmessage:sub(first + 1, last - 1)
    local after = newmessage:sub(last + 1)
    local price = pricing.seen[during] and pricing.pretty(pricing.seen[during])
    if price then
      newmessage = before .. price .. after
    else
      problems[#problems + 1] = during
      newmessage = before .. "N/A" .. after
    end
  end
  if #problems > 0 then
    local error = sprintf("Missing: %s", table.concat(problems, ", "))
    for k, v in pairs(pricing.buttons) do
      pcall(pricing.error_button, k, error)
    end
  else
    -- printf("Updated: %s", newmessage)
    pricing.last_computed = newmessage
    for k, v in pairs(pricing.buttons) do
      pcall(pricing.set_button, k, newmessage)
    end
  end
end

function pricing.set_button(i, newmessage)
  pricing.buttons[i]:EventMacroSet(Event.UI.Input.Mouse.Left.Click, tostring(i) .. " " .. newmessage)
end

function pricing.error_button(i, newmessage)
  pricing.buttons[i]:EventMacroSet(Event.UI.Input.Mouse.Left.Click, sprintf('script dump("%s")', newmessage:gsub('"', "''")))
end

function pricing.do_updates(event)
  pricing.pending = false
  for k, v in pairs(Pricing_sv.items) do
    LibPGCEx.GetPrices(function(...) pricing.get_update(k, ...) end, v, nil, 'BMarket')
  end
end

function pricing.movebutton()
  local tb = Pricing_sv.loc.y < 0 and "BOTTOM" or "TOP"
  local lr = Pricing_sv.loc.x < 0 and "RIGHT" or "LEFT"
  local point = tb .. lr
  pricing.updatebutton:ClearPoint("TOPLEFT")
  pricing.updatebutton:ClearPoint("BOTTOMLEFT")
  pricing.updatebutton:ClearPoint("TOPRIGHT")
  pricing.updatebutton:ClearPoint("BOTTOMRIGHT")
  pricing.updatebutton:SetPoint(point, UIParent, point, Pricing_sv.loc.x, Pricing_sv.loc.y)
end

function pricing.setup(event)
  pricing.buttoncontext = UI.CreateContext('Pricing')
  pricing.buttoncontext:SetStrata('dialog')
  pricing.buttoncontext:SetSecureMode("restricted")
  pricing.tipcontext = UI.CreateContext('Pricing')
  pricing.tipcontext:SetStrata('topmost')
  pricing.updatebutton = UI.CreateFrame('RiftButton', 'update price list', pricing.buttoncontext)
  pricing.movebutton()
  pricing.updatebutton:SetWidth(100)
  pricing.updatebutton:EventAttach(Event.UI.Input.Mouse.Left.Click, pricing.do_updates, "update button")
  local text = UI.CreateFrame('Text', 'button label', pricing.updatebutton)
  text:SetPoint("CENTER", pricing.updatebutton, "CENTER")
  text:SetText('UPDATE')
  for k, v in pairs(Pricing_sv.channels) do
    pricing.buttons[v] = pricing.newbutton(k, v)
    pricing.buttons[v]:EventMacroSet(Event.UI.Input.Mouse.Left.Click, "script dump('No data seen yet.')")
  end
end

function pricing.maybe_update(event, etype, auctions)
  if not pricing.pending then
    if etype and etype.type == 'search' then
      pricing.pending = true
      printf("AH search data showed up. Will update automatically in about 10 seconds.")
      Library.LibCron.new('Pricing', 10, false, false, pricing.do_updates)
    end
  end
end

function pricing.find_new_key(key, name, quality, results)
  local found = false
  for k, v in pairs(results) do
    local seen = {}
    if v.itemName == name and ((v.itemRarity == quality) or (not quality)) then
      found = true
      if key then
        Pricing_sv.items[key] = v.itemType
        printf("Found addon type for %s-quality %s, saving as key %s.",
          quality, name, key)
        break
      else
	if not seen[v.itemRarity] then
          LibPGCEx.GetPrices(function(...) pricing.display_update(name .. "[" .. v.itemRarity .. "]", ...) end, v.itemType, nil, 'BMarket')
	  seen[v.itemRarity] = true
        end
      end
    end
  end
  if not found then
    printf("Found no instances of %s-quality %s.", quality, name)
  end
end

function pricing.slashcommand(args)
  local okay = false
  if args then
    if args.p then
      local x, y = args.p:match('([-%d]*),([-%d]*)')
      if x and y then
        x = tonumber(x)
        y = tonumber(y)
        Pricing_sv.loc = { x = x, y = y }
        pricing.movebutton()
	okay = true
      end
    elseif args.c then
      local found = false
      if args.c:sub(1, 1) == '~' then
        local channel = args.c:sub(2)
	for i = 1, #Pricing_sv.channels do
	  if Pricing_sv.channels[i] == channel then
	    found = i
	    break
	  end
	end
	if found then
	  table.remove(Pricing_sv.channels, found)
	else
	  printf("Don't seem to have a channel '%s' to delete.", channel)
	end
      else
	for i = 1, #Pricing_sv.channels do
	  if Pricing_sv.channels[i] == args.c then
	    found = i
	    break
	  end
	end
	if found then
	  printf("Already got a button for '%s'. Maybe you wanted '~%s'?",
	    args.c, args.c)
	else
	  Pricing_sv.channels[#Pricing_sv.channels + 1] = args.c
	  printf("Adding new channel '%s'.", args.c)
	end
      end
      if found then
	okay = true
        printf("Changes will take effect on next load.")
      end
    elseif args.d and args.k then
      if Pricing_sv.items[args.k] then
        okay = true
        Pricing_sv.items[args.k] = nil
        printf("Deleting key '%s'.", args.k) 
      else
        printf("No existing key '%s'.", args.k)
      end
    elseif args.i then
      okay = true
      printf("Attemping search for %s quality %s. If this doesn't do anything, make sure BananAH has current scan data.", args.q or "any", args.i)
      LibPGC.SearchAuctions(function(t) pricing.find_new_key(args.k, args.i, args.q, t) end, nil, args.q, nil, nil, nil, nil, nil, args.i)
    elseif args.m then
      okay = true
      Pricing_sv.message = args.leftover_full
      printf("New message: %s", Pricing_sv.message)
    elseif args.l then
      okay = true
      printf("Keys:")
      for k, v in pairs(Pricing_sv.items) do
	local status, value = pcall(Inspect.Item.Detail, v)
	if status then
	  printf("  %s (%s-quality %s)", k, value.rarity, value.name)
	  printf("     [%s]", value.id)
	else
          printf("  %s unavailable [%s]", k, v)
	end
      end
      printf("Channels:")
      for i = 1, #Pricing_sv.channels do
        printf("  %d: %s", i, Pricing_sv.channels[i])
      end
      printf("Message:")
      printf("  %s", Pricing_sv.message)
      if pricing.last_computed then
        printf("  %s", pricing.last_computed)
      end
    end
  end
  if not okay then
    printf("Usage: /pricing [-l] [-p x,y] [-d -k key] [-k key -q quality -i item] [-m message] [-c channel/-c ~channel]")
    printf("Positive values are from top/left, negative from bottom/right. No spaces, please.")
    printf("-d to delete a key, -q/-i to create one, -m to set a new message.")
    printf("-c # to add a channel, -c ~# to delete it.")
  end
end

function pricing.tooltip_update(data)
  local maxwidth = 0
  local counter = 0
  local price = nil
  if data and data.BMarket and data.BMarket.buy then
    price = data.BMarket.buy
  end
  local pretty
  if price then
    pretty = pricing.prettyverbose(price, "Value")
  else
    pretty = "No AH data."
  end
  counter = counter + 1
  if counter > #pricing.labels then
    printf("Creating label.")
    table.insert(pricing.labels, UI.CreateFrame('Text', sprintf("pricing_label_%d", counter), pricing.subframe))
    pricing.labels[counter]:SetPoint("TOPRIGHT", pricing.labels[counter - 1], "BOTTOMRIGHT", 0, 1)
    pricing.labels[counter]:SetFontSize(pricing.labels[counter]:GetFontSize() + 2)
    printf("Created label.")
  end
  pricing.labels[counter]:SetText(pretty, true)
  pricing.labels[counter]:SetFontColor(1, 1, 1)
  pricing.labels[counter]:SetVisible(true)
  local width = pricing.labels[counter]:GetWidth()
  if width > maxwidth then
    maxwidth = width
  end
  pricing.subframe:SetPoint("BOTTOM", pricing.labels[counter], "BOTTOM", nil, -2)
  pricing.subframe:SetWidth(maxwidth + 4)
  if counter < #pricing.labels then
    for i = counter + 1, #pricing.labels do
      pricing.labels[i]:SetVisible(false)
    end
  end
  return true
end

function pricing.show_tooltip(item, data)
  if item ~= pricing.show_this_item then
    return
  end
  if not pricing.tooltip then
    pricing.labels = {}
    pricing.tooltip = UI.CreateFrame('Frame', 'pricing_tooltip_frame', pricing.tipcontext)
    if not pricing.tooltip then
      printf("Couldn't create a tooltip.  Oops.")
      return
    end
    pricing.tooltip:SetBackgroundColor(0.4, 0.4, 0.3, 1.0)
    pricing.subframe = UI.CreateFrame('Frame', 'pricing_tooltip_subframe', pricing.tooltip)
    pricing.subframe:SetBackgroundColor(0.1, 0.1, 0.1, 1.0)
    table.insert(pricing.labels, UI.CreateFrame('Text', 'pricing_tooltip_1', pricing.subframe))

    pricing.tooltip:SetPoint("TOPRIGHT", UI.Native.Tooltip, "TOPLEFT", -5, 5)
    pricing.tooltip:SetPoint("BOTTOMLEFT", pricing.subframe, "BOTTOMLEFT", -1, 1)
    pricing.subframe:SetPoint("TOPRIGHT", pricing.tooltip, "TOPRIGHT", -1, 1)
    pricing.labels[1]:SetPoint("TOPRIGHT", pricing.subframe, "TOPRIGHT", -2, -2)
    pricing.labels[1]:SetFontSize(pricing.labels[1]:GetFontSize() + 2)
    pricing.labels[1]:SetText("")
  end
  pricing.tooltip:SetVisible(pricing.tooltip_update(data))
  local point, offset
  -- This attempts to ensure a little margin near edge of screen; note that
  -- the visible native tooltip is not the actual height of the window
  local width = pricing.tooltip:GetWidth()
  if UI.Native.Tooltip:GetLeft() - width > 0 then
    point = "TOPLEFT"
    offset = -5
  else
    point = "TOPRIGHT"
    offset = 5 + width
  end
  -- printf("Native's left %d, my width %d, point %s, offset %d.", UI.Native.Tooltip:GetLeft(), width, point, offset)
  local status, value = pcall(pricing.tooltip.SetPoint, pricing.tooltip, "TOPRIGHT", UI.Native.Tooltip, point, offset, 10)
  if not status then
    printf("Error: %s", value)
  end
end

function pricing.hide_tooltip()
  if pricing.tooltip then
    pricing.tooltip:SetVisible(false)
  end
  Command.Event.Detach(Event.System.Update.Begin, nil, 'pricing_tooltip')
end

function pricing.soon_tooltip(itype)
  if Inspect.Time.Frame() == pricing.this_frame then
    return
  else
    -- turn ourselves off...
    Command.Event.Detach(Event.System.Update.Begin, nil, 'pricing_tooltip')
    LibPGCEx.GetPrices(function(...) pricing.show_tooltip(itype, ...) end, itype, nil, nil)
  end
end

function pricing.tooltip_handler(event, tt_type, shown, buff)
  if tt_type == 'item' or tt_type == 'itemtype' then
    pricing.this_frame = Inspect.Time.Frame()
    local ok, details = pcall(function() return Inspect.Item.Detail(shown) end)
    if details and details.type and not details.bound then
      pricing.show_this_item = details.type
      Command.Event.Detach(Event.System.Update.Begin, nil, 'pricing_tooltip')
      Command.Event.Attach(Event.System.Update.Begin, function() pricing.soon_tooltip(details.type) end, 'pricing_tooltip')
    else
      if not ok then
        printf("Error: %s", details)
      end
      pricing.hide_tooltip()
    end
  else
    if Inspect.Time.Frame() ~= pricing.this_frame then
      pricing.show_this_item = nil
      pricing.hide_tooltip()
    end
  end
end


Command.Event.Attach(Event.Tooltip, pricing.tooltip_handler, "tooltip handler")
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, pricing.variables, "variable check")
Command.Event.Attach(Event.Addon.Startup.End, pricing.setup, "initial setup")
Command.Event.Attach(Event.Auction.Scan, pricing.maybe_update, "consider updating")
Library.LibGetOpt.makeslash("c:di:k:lmp:q:", "Pricing", "pricing", pricing.slashcommand)
